@@ -10,9 +10,37 @@ TableInfo = namedtuple("TableInfo", ['checks', 'relkind', 'hasindex',
 log = logging.getLogger(__name__)
 
 
-@special_command('\\l', '\\l', 'List databases.', arg_type=RAW_QUERY)
-def list_databases(cur, **_):
-    query = 'SELECT datname FROM pg_database;'
+@special_command('\\l', '\\l[+] [pattern]', 'List databases.', aliases=('\\list',))
+def list_databases(cur, pattern, verbose):
+    query = '''SELECT d.datname as "Name",
+        pg_catalog.pg_get_userbyid(d.datdba) as "Owner",
+        pg_catalog.pg_encoding_to_char(d.encoding) as "Encoding",
+        d.datcollate as "Collate",
+        d.datctype as "Ctype",
+        pg_catalog.array_to_string(d.datacl, E'\n') AS "Access privileges"'''
+    if verbose:
+        query += ''',
+            CASE WHEN pg_catalog.has_database_privilege(d.datname, 'CONNECT')
+                    THEN pg_catalog.pg_size_pretty(pg_catalog.pg_database_size(d.datname))
+                    ELSE 'No Access'
+            END as "Size",
+            t.spcname as "Tablespace",
+            pg_catalog.shobj_description(d.oid, 'pg_database') as "Description"'''
+    query += '''
+    FROM pg_catalog.pg_database d
+    '''
+    if verbose:
+        query += '''
+        JOIN pg_catalog.pg_tablespace t on d.dattablespace = t.oid
+        '''
+    params = []
+    if pattern:
+        query += '''
+        WHERE d.datname ~ %s
+        '''
+        _, schema = sql_name_pattern(pattern)
+        params.append(schema)
+    query = cur.mogrify(query + ' ORDER BY 1', params)
     cur.execute(query)
     if cur.description:
         headers = [x[0] for x in cur.description]
@@ -924,6 +952,7 @@ def describe_one_table_details(cur, schema_name, relation_name, oid, verbose):
 
                 #/* If exclusion constraint, print the constraintdef */
                 if row[7] == "x":
+                    status.append(' ')
                     status.append(row[6])
                 else:
                     #/* Label as primary key or unique (but not both) */
@@ -1149,7 +1178,7 @@ def describe_one_table_details(cur, schema_name, relation_name, oid, verbose):
     #*/
     if (tableinfo.relkind == 'r' or tableinfo.relkind == 'm' or
             tableinfo.relkind == 'f'):
-        # /* print foreign server name */
+        #/* print foreign server name */
         if tableinfo.relkind == 'f':
             #/* Footer information about foreign table */
             sql = ("SELECT s.srvname,\n"
@@ -1164,10 +1193,10 @@ def describe_one_table_details(cur, schema_name, relation_name, oid, verbose):
             cur.execute(sql)
             row = cur.fetchone()
 
-            # /* Print server name */
+            #/* Print server name */
             status.append("Server: %s\n" % row[0])
 
-            # /* Print per-table FDW options, if any */
+            #/* Print per-table FDW options, if any */
             if (row[1]):
                 status.append("FDW Options: (%s)\n" % row[1])
 
@@ -1211,16 +1240,20 @@ def describe_one_table_details(cur, schema_name, relation_name, oid, verbose):
             #/* print the number of child tables, if any */
             if (cur.rowcount > 0):
                 status.append("Number of child tables: %d (Use \d+ to list"
-                    "them.)\n" % cur.rowcount)
+                              " them.)\n" % cur.rowcount)
         else:
-            spacer = ''
-            if (cur.rowcount >0):
+            if (cur.rowcount > 0):
                 status.append('Child tables')
 
-            #/* display the list of child tables */
-            for row in cur:
-                status.append("%s: %s,\n" % (spacer, row))
-                spacer = ' ' * len('Child tables')
+                spacer = ':'
+                trailer = ',\n'
+                #/* display the list of child tables */
+                for idx, row in enumerate(cur, 1):
+                    if idx == 2:
+                        spacer = ' ' * (len('Child tables') + 1)
+                    if idx == cur.rowcount:
+                        trailer = '\n'
+                    status.append("%s %s%s" % (spacer, row[0], trailer))
 
         #/* Table type */
         if (tableinfo.reloftype):
